import 'package:flutter/material.dart';

class PumaGuideItem {
  final GlobalKey? globalKey;
  final ShapeFocus? shapeFocus;
  final Widget Function(Function skip, Function next, int index, int total)
      builder;
  final double? top;
  final double? bottom;
  final double? left;
  final double? right;
  final CrossAxisAlignment crossAxisAlignment;
  final MainAxisAlignment mainAxisAlignment;
  final bool? touchScreen;

  PumaGuideItem({
    this.top,
    this.bottom,
    this.left,
    this.right,
    this.globalKey,
    required this.builder,
    this.shapeFocus,
    this.crossAxisAlignment = CrossAxisAlignment.center,
    this.mainAxisAlignment = MainAxisAlignment.center,
    this.touchScreen = false,
  });
}

class PumaGuide {
  static start(BuildContext context, List<PumaGuideItem> children) async {
    int count = 0;
    var size = MediaQuery.of(context).size;
    OverlayState? overlayState = Overlay.of(context);
    List<OverlayEntry> entrys = [];
    children.forEach((element) async {
      var offset = _capturePositionWidget(element.globalKey);
      var sizeWidget = _getSizeWidget(element.globalKey);
      entrys.add(
        OverlayEntry(
          builder: (context) {
            final next = () {
              entrys[count].remove();
              count++;
              if (count != entrys.length) {
                overlayState?.insert(entrys[count]);
              }
            };
            final skip = () {
              entrys[count].remove();
            };
            return GestureDetector(
              onTap: element.touchScreen == true ? next : () {},
              child: Scaffold(
                backgroundColor: Colors.transparent,
                body: Stack(
                  children: [
                    CustomPaint(
                      size: size,
                      painter: HolePainter(
                          shapeFocus: element.shapeFocus,
                          dx: offset.dx + (sizeWidget.width / 2),
                          dy: offset.dy + (sizeWidget.height / 2),
                          width: sizeWidget.width,
                          height: sizeWidget.height),
                    ),
                    Positioned(
                      top: element.top,
                      bottom: element.bottom,
                      left: element.left,
                      right: element.right,
                      child: Container(
                        width: size.width * 0.8,
                        child: Column(
                          crossAxisAlignment: element.crossAxisAlignment,
                          mainAxisAlignment: element.mainAxisAlignment,
                          children: [
                            element.builder(skip, next, count, children.length),
                          ],
                        ),
                      ),
                    )
                  ],
                ),
              ),
            );
          },
        ),
      );
    });

    overlayState?.insert(entrys[0]);
  }

  static Offset _capturePositionWidget(GlobalKey? key) {
    RenderBox? renderPosition =
        key!.currentContext!.findRenderObject() as RenderBox;

    return renderPosition.localToGlobal(Offset.zero);
  }

  static Size _getSizeWidget(GlobalKey? key) {
    RenderBox? renderSize =
        key!.currentContext?.findRenderObject() as RenderBox;
    return renderSize.size;
  }
}

class HolePainter extends CustomPainter {
  final double? dx;
  final double? dy;
  final double? width;
  final double? height;
  final ShapeFocus? shapeFocus;

  HolePainter({
    this.dx,
    this.dy,
    this.width,
    this.height,
    this.shapeFocus = ShapeFocus.oval,
  });
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()..color = Color.fromRGBO(0, 0, 0, 0.8);
    if (shapeFocus == ShapeFocus.oval) {
      canvas.drawPath(
          Path.combine(
            PathOperation.difference,
            Path()..addRect(Rect.fromLTWH(0, 0, size.width, size.height)),
            Path()
              ..addOval(
                  Rect.fromCircle(center: Offset(dx!, dy!), radius: width!))
              ..close(),
          ),
          paint);
    } else {
      canvas.drawPath(
          Path.combine(
            PathOperation.difference,
            Path()..addRect(Rect.fromLTWH(0, 0, size.width, size.height)),
            Path()
              ..addRect(Rect.fromLTWH(
                  dx! - (width! / 2), dy! - (height! / 2), width!, height!))
              ..close(),
          ),
          paint);
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}

enum ShapeFocus { oval, square }
