import 'package:built_collection/built_collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/helpers.dart' as Pre;
import 'package:puma_ui/redux/uiState.dart';
import 'package:redux/redux.dart';

class ApiButton<T extends VirtualUiState> extends StatefulWidget {
  final Function apiCall;
  final Function? finishCallback;
  final String label;
  final Color color;
  final bool disabled;
  const ApiButton({
    required this.apiCall,
    this.label = "Label",
    this.finishCallback,
    this.color = Colors.blue,
    this.disabled = false,
  });

  @override
  _ApiButtonState createState() => _ApiButtonState<T>();
}

class _ApiButtonState<T extends VirtualUiState> extends State<ApiButton> {
  String completionKey = "";
  bool processing = false;
  @override
  Widget build(BuildContext context) {
    return (StoreConnector<T, _ViewModel>(
        onDidChange: (_ViewModel? vm, _ViewModel a) {
          if (processing &&
              vm!.networkCompletionStack[completionKey] == null &&
              widget.finishCallback != null) {
            widget.finishCallback!();
            processing = false;
          }
        },
        converter: (Store<T> store) => _ViewModel.create(store),
        builder: (BuildContext context, _ViewModel viewModel) {
          return Opacity(
              opacity: widget.disabled ? 0.3 : 1,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  ElevatedButton(
                    style: ButtonStyle(
                      backgroundColor: MaterialStateProperty.all(widget.color),
                      textStyle: MaterialStateProperty.all(
                        TextStyle(color: Colors.white),
                      ),
                    ),
                    onPressed: widget.disabled
                        ? () => {print("disabled")}
                        : () {
                            var call = widget.apiCall();
                            processing = true;
                            if (call != null)
                              completionKey =
                                  viewModel.withNetworkCompletionDispatch(call);
                          },
                    child: Row(
                      children: [
                        if (viewModel.networkCompletionStack[completionKey] ??
                            false)
                          SizedBox(
                            child: CircularProgressIndicator(
                                backgroundColor: Colors.white),
                            height: 20.0,
                            width: 20.0,
                          ),
                        Padding(
                          padding: const EdgeInsets.only(left: 8.0),
                          child: Text(
                            widget.label,
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ));
        }));
  }
}

class _ViewModel {
  final BuiltMap<String, bool> networkCompletionStack;
  final Function withNetworkCompletionDispatch;
  _ViewModel(
    this.networkCompletionStack,
    this.withNetworkCompletionDispatch,
  );

  factory _ViewModel.create(Store<VirtualUiState> store) {
    return _ViewModel(store.state.networkState.networkCompletionStack,
        (f) => Pre.withNetworkCompletionDispatch(store, f));
  }
}
